@@ -224,6 +224,15 @@ class MemorizedResult(Logger):
         self.verbose = verbose
         self.timestamp = timestamp
 
+    @property
+    def argument_hash(self):
+        warnings.warn(
+            "The 'argument_hash' attribute has been deprecated in version "
+            "0.12 and will be removed in version 0.14.\n"
+            "Use `args_id` attribute instead.",
+            DeprecationWarning, stacklevel=2)
+        return self.args_id
+
     def get(self):
         """Read value from cache and return it."""
         if self.verbose:
@@ -241,13 +250,12 @@ class MemorizedResult(Logger):
 
     def __repr__(self):
         return ('{class_name}(location="{location}", func="{func}", '
-                'argument_hash="{argument_hash}")'
+                'args_id="{args_id}")'
                 .format(class_name=self.__class__.__name__,
                         location=self.store_backend,
                         func=self.func,
-                        argument_hash=self.args_id
+                        args_id=self.args_id
                         ))
-
     def __reduce__(self):
         return (self.__class__,
                 (self.store_backend, self.func, self.args_id),
@@ -824,8 +832,8 @@ class Memory(Logger):
             warnings.warn(
                 "The 'cachedir' parameter has been deprecated in version "
                 "0.12 and will be removed in version 0.14.\n"
-                'You provided "cachedir={!r}", '
-                'use "location={!r}" instead.'.format(cachedir, location),
+                'You provided "cachedir={0!r}", '
+                'use "location={0!r}" instead.'.format(cachedir),
                 DeprecationWarning, stacklevel=2)
             location = cachedir
 
