@@ -17,6 +17,7 @@ import threading
 import itertools
 from numbers import Integral
 import warnings
+from functools import partial
 
 from ._multiprocessing_helpers import mp
 
@@ -59,7 +60,8 @@ VALID_BACKEND_CONSTRAINTS = ('sharedmem', None)
 def _register_dask():
     """ Register Dask Backend if called with parallel_backend("dask") """
     try:
-        import distributed.joblib  # noqa: #F401
+        from ._dask import DaskDistributedBackend
+        register_parallel_backend('dask', DaskDistributedBackend)
     except ImportError:
         msg = ("To use the dask.distributed backend you must install both "
                "the `dask` and distributed modules.\n\n"
@@ -209,14 +211,49 @@ class CloudpickledObjectWrapper(object):
         return loads, (self.pickled_obj,)
 
 
+def _need_pickle_wrapping(obj):
+    if isinstance(obj, list) and len(obj) >= 1:
+        # Make the assumption that the content of the list is homogeneously
+        # typed.
+        return _need_pickle_wrapping(obj[0])
+    elif isinstance(obj, dict) and len(obj) >= 1:
+        # Make the assumption that the content of the dict is homogeneously
+        # typed.
+        k, v = next(iter(obj.items()))
+        return _need_pickle_wrapping(v) or _need_pickle_wrapping(k)
+    elif isinstance(obj, partial):
+        return _need_pickle_wrapping(obj.func)
+
+    # Warning: obj.__module__ can be defined and set to None
+    module = getattr(obj, "__module__", None)
+    need_wrap = module is not None and "__main__" in module
+    if callable(obj):
+        # Need wrap if the object is a function defined in a local scope of
+        # another function.
+        func_code = getattr(obj, "__code__", "")
+        need_wrap |= getattr(func_code, "co_flags", 0) & inspect.CO_NESTED
+
+        # Need wrap if the obj is a lambda expression
+        func_name = getattr(obj, "__name__", "")
+        need_wrap |= "<lambda>" in func_name
+
+        # Need wrap if obj is a bound method of an instance of an
+        # interactively defined class
+        method_self = getattr(obj, '__self__', None)
+        if not need_wrap and method_self is not None:
+            # Recursively introspect the instanc of the method
+            return _need_pickle_wrapping(method_self)
+    return need_wrap
+
+
 class BatchedCalls(object):
     """Wrap a sequence of (func, args, kwargs) tuples as a single callable"""
 
-    def __init__(self, iterator_slice, backend, pickle_cache):
+    def __init__(self, iterator_slice, backend, pickle_cache=None):
         self.items = list(iterator_slice)
         self._size = len(self.items)
         self._backend = backend
-        self._pickle_cache = pickle_cache
+        self._pickle_cache = pickle_cache if pickle_cache is not None else {}
 
     def __call__(self):
         with parallel_backend(self._backend):
@@ -228,24 +265,19 @@ class BatchedCalls(object):
 
     @staticmethod
     def _wrap_non_picklable_objects(obj, pickle_cache):
-        need_wrap = "__main__" in getattr(obj, "__module__", "")
-        if callable(obj):
-            # Need wrap if the object is a function defined in a local scope of
-            # another function.
-            func_code = getattr(obj, "__code__", "")
-            need_wrap |= getattr(func_code, "co_flags", 0) & inspect.CO_NESTED
-
-            # Need wrap if the obj is a lambda expression
-            func_name = getattr(obj, "__name__", "")
-            need_wrap |= "<lambda>" in func_name
-
-        if not need_wrap:
+        if not _need_pickle_wrapping(obj):
             return obj
-
-        wrapped_obj = pickle_cache.get(obj)
+        try:
+            wrapped_obj = pickle_cache.get(obj)
+            hashable = True
+        except TypeError:
+            # obj is not hashable: cannot be cached
+            wrapped_obj = None
+            hashable = False
         if wrapped_obj is None:
             wrapped_obj = CloudpickledObjectWrapper(obj)
-            pickle_cache[obj] = wrapped_obj
+            if hashable:
+                pickle_cache[obj] = wrapped_obj
         return wrapped_obj
 
     def __getstate__(self):
