@@ -451,6 +451,19 @@ def test_call_and_shelve(tmpdir):
         result.clear()  # Do nothing if there is no cache.
 
 
+def test_call_and_shelve_argument_hash(tmpdir):
+    # Verify that a warning is raised when accessing arguments_hash
+    # attribute from MemorizedResult
+    func = Memory(location=tmpdir.strpath, verbose=0).cache(f)
+    result = func.call_and_shelve(2)
+    assert isinstance(result, MemorizedResult)
+    with warns(DeprecationWarning) as w:
+        assert result.argument_hash == result.args_id
+    assert len(w) == 1
+    assert "The 'argument_hash' attribute has been deprecated" \
+        in str(w[-1].message)
+
+
 def test_memorized_pickling(tmpdir):
     for func in (MemorizedFunc(f, tmpdir.strpath), NotMemorizedFunc(f)):
         filename = tmpdir.join('pickling_test.dat').strpath
@@ -586,6 +599,16 @@ def test_clear_memory_with_none_location():
 
 
 if PY3_OR_LATER:
+    # Avoid flake8 F821 "undefined name" warning. func_with_kwonly_args and
+    # func_with_signature are redefined in the exec statement a few lines below
+    def func_with_kwonly_args():
+        pass
+
+    def func_with_signature():
+        pass
+
+    # exec is needed to define a function with a keyword-only argument and a
+    # function with signature while avoiding a SyntaxError on Python 2
     exec("""
 def func_with_kwonly_args(a, b, *, kw1='kw1', kw2='kw2'):
     return a, b, kw1, kw2
